Option Explicit

'VB Sprite Engine, Copyright Mark Meany 1997.
'This Sprite Engine is FreeWare, use as you like for what you like!

'Module Name    Sprite Engine
'Copyright      Mark Meany, 1997
'Version        v1.0
'Release Date   1st September 1997

'Know Bugs

'Revision History

'=========================================================
'User defined types

'Frame structure
Type SprFrame
    iX As Integer               'X pixel coord of frame TopLeft
    iY As Integer               'Y pixel coord of frame TopLeft
    iW As Integer               'Pixel width of frame
    iH As Integer               'Pixel height of frame
End Type

'Sprite structure
Type SprSprite
    iInUse As Integer           'True if sprite is allocated
    iActive As Integer          'True if sprite is displayed
    iX As Integer               'X position at which last drawn
    iY As Integer               'Y position at which last drawn
    iW As Integer               'Pixel width of sprite
    iH As Integer               'Pixel height of sprite
    iFrame As Integer           'Current frame index into gtSprFrm()
    iFrameX As Integer          'X offset into sprite graphics DC
    iFrameY As Integer          'Y offset into sprite graphics DC
    iSaveDC As Integer          'DC for background saves
    iSaveBmp As Integer         'Bitmap for background saves
    iFirstFrame As Integer      'Index of first frame in anim sequence
    iLastFrame As Integer       'Index of last frame in anim sequence
    iAnimAuto As Integer        'True if animation is controlled by Sprite Engine
    iAnimRate As Integer        'Speed of animation (cycles per frame)
    iAnimCount As Integer       'Down counter until next frame of anim
    iUsr1 As Integer            'User data
    iUsr2 As Integer            'User data
    iUsr3 As Integer            'User data
End Type

'New Sprite Structure
Type SprNewSprite
    iId As Integer              'Index of sprite to allocate
    iFirstFrame As Integer      'First Anim Sequence frame
    iLastFrame As Integer       'Last Anim Sequence frame
    iAnimFlag As Integer        'True if Auto animated sprite
    iAnimRate As Integer        'Animation Speed
End Type

'=========================================================
'Constants

'=========================================================
'Global Variables

Global gtSprFrm() As SprFrame   'Holds all frame details
Global gtSpr() As SprSprite     'Holds all sprite details

'=========================================================
'Module Variables
Dim miGameDC As Integer         'memory DC for game play area
Dim miGameBmp As Integer        'Bitmap for game play area
Dim miGameW As Integer          'Pixel width of play area
Dim miGameH As Integer          'Pixel height of play area
Dim miGfxDC As Integer              'DC for sprite graphics data
Dim miGfxBmp As Integer             'Bitamp for sprite graphics data
Dim miMaskDC As Integer             'DC for sprite mask data
Dim miMaskBmp As Integer            'Bitmap for sprite mask data

'=========================================================
'API Constants & Types

'Constants for BitBlt() copy modes
Global Const SRCCOPY = &HCC0020
Global Const SRCAND = &H8800C6
Global Const SRCPAINT = &HEE0086
Global Const NOTSRCCOPY = &H330008
Global Const SRCERASE = &H440328
Global Const SRCINVERT = &H660046

'Constants for objects Scale Mode
Global Const TWIPS = 1
Global Const PIXELS = 3
Global Const RES_INFO = 2
Global Const MINIMIZED = 1

'=========================================================
'API Function Imports

'Windows resources functions
Declare Function BitBlt Lib "GDI" (ByVal hDestDC As Integer, ByVal X As Integer, ByVal Y As Integer, ByVal nWidth As Integer, ByVal nHeight As Integer, ByVal hSrcDC As Integer, ByVal XSrc As Integer, ByVal YSrc As Integer, ByVal dwRop As Long) As Integer
Declare Function CreateCompatibleDC Lib "GDI" (ByVal hDC As Integer) As Integer
Declare Function CreateBitmap Lib "GDI" (ByVal nWidth As Integer, ByVal nHeight As Integer, ByVal nPlanes As Integer, ByVal nBitCount As Integer, ByVal lpBits As Any) As Integer
Declare Function CreateCompatibleBitmap Lib "GDI" (ByVal hDC As Integer, ByVal nWidth As Integer, ByVal nHeight As Integer) As Integer
Declare Function SelectObject Lib "GDI" (ByVal hDC As Integer, ByVal hObject As Integer) As Integer
Declare Function DeleteObject Lib "GDI" (ByVal hObject As Integer) As Integer
Declare Function DeleteDC Lib "GDI" (ByVal hDC As Integer) As Integer
Declare Function SetBkColor Lib "GDI" (ByVal hDC As Integer, ByVal crColor As Long) As Long
Declare Function sndPlaySound Lib "MMSystem" (lpsound As Any, ByVal flag As Integer) As Integer

'Windows resources functions for VB 4 users
'Declare Function BitBlt Lib "gdi32" (ByVal hDestDC As Long, ByVal x As Long, ByVal y As Long, ByVal nWidth As Long, ByVal nHeight As Long, ByVal hSrcDC As Long, ByVal xSrc As Long, ByVal ySrc As Long, ByVal dwRop As Long) As Long
'Declare Function CreateCompatibleDC Lib "gdi32" (ByVal hdc As Long) As Long
'Declare Function CreateBitmap Lib "gdi32" (ByVal nWidth As Long, ByVal nHeight As Long, ByVal nPlanes As Long, ByVal nBitCount As Long, lpBits As Any) As Long
'Declare Function CreateCompatibleBitmap Lib "gdi32" (ByVal hdc As Long, ByVal nWidth As Long, ByVal nHeight As Long) As Long
'Declare Function SelectObject Lib "gdi32" (ByVal hdc As Long, ByVal hObject As Long) As Long
'Declare Function DeleteObject Lib "gdi32" (ByVal hObject As Long) As Long
'Declare Function DeleteDC Lib "gdi32" (ByVal hdc As Long) As Long
'Declare Function SetBkColor Lib "gdi32" (ByVal hdc As Long, ByVal crColor As Long) As Long
'Declare Function sndPlaySound Lib "winmm.dll" Alias "sndPlaySoundA" (ByVal lpszSoundName As String, ByVal uFlags As Long) As Long

'=========================================================
'Purpose
'Entry
'Exit
'Comments



'=========================================================
'Purpose
'Entry
'Exit
'Comments

Function iSprAllocateSprite (rtNew As SprNewSprite) As Integer

'=========================================================
'Purpose    Allocate a sprite from the system, creates a
'           Save DC for background saving and initialises
'           all fields of the Sprite structure based on
'           values passed in the New Sprite structure.
'Entry      rtNew - A SprNewSprite structure that defines
'                   the sprite to allocate.
'Exit       True if sprite allocated successfully
'Comments   Resources are freed should an error occur

Dim i As Integer
Dim iX As Integer
Dim iY As Integer
Dim iW As Integer
Dim iH As Integer
Dim iSaveDC As Integer
Dim iSaveBmp As Integer
Dim iRetVal As Integer

'If there is no play area DC then exit
If miGameDC = 0 Then Exit Function

'If the requested sprite is already allocated then exit
If gtSpr(rtNew.iId).iInUse Then Exit Function

'Get some values to work with
iX = gtSprFrm(rtNew.iFirstFrame).iX
iY = gtSprFrm(rtNew.iFirstFrame).iY
iW = gtSprFrm(rtNew.iFirstFrame).iW
iH = gtSprFrm(rtNew.iFirstFrame).iH

'Allocate a save DC based on size of first animation frame
iSaveDC = CreateCompatibleDC(miGameDC)

'Allocate a save Bitmap based on size of first animation frame
If iSaveDC Then
    iSaveBmp = CreateCompatibleBitmap(miGameDC, iW, iH)
    If iSaveBmp Then
	iSaveBmp = SelectObject(iSaveDC, iSaveBmp)

	'Fill in fields of gtSpr() for this sprite and flag success
	gtSpr(rtNew.iId).iInUse = True
	gtSpr(rtNew.iId).iActive = False
	gtSpr(rtNew.iId).iX = 0
	gtSpr(rtNew.iId).iY = 0
	gtSpr(rtNew.iId).iW = iW
	gtSpr(rtNew.iId).iH = iH
	gtSpr(rtNew.iId).iFrame = rtNew.iFirstFrame
	gtSpr(rtNew.iId).iFrameX = iX
	gtSpr(rtNew.iId).iFrameY = iY
	gtSpr(rtNew.iId).iSaveDC = iSaveDC
	gtSpr(rtNew.iId).iSaveBmp = iSaveBmp
	gtSpr(rtNew.iId).iFirstFrame = rtNew.iFirstFrame
	gtSpr(rtNew.iId).iLastFrame = rtNew.iLastFrame
	gtSpr(rtNew.iId).iAnimAuto = rtNew.iAnimFlag
	gtSpr(rtNew.iId).iAnimRate = rtNew.iAnimRate
	gtSpr(rtNew.iId).iAnimCount = rtNew.iAnimRate
	iRetVal = True
    Else
	'Error allocating bitmap, free DC
	i = DeleteDC(iSaveDC)
    End If
End If

'Return success
iSprAllocateSprite = iRetVal

End Function

Function iSprCollision (ByVal viId1 As Integer, ByVal viId2 As Integer) As Integer

'=========================================================
'Purpose    To determine if two sprites have collided
'Entry      viId1 - Index of first sprite
'           viId2 - Index of second sprite
'Exit       True if sprites have hit
'Comments   This uses a crude overlapping rectangles
'           algorithm that causes yells of 'that missed
'           me by miles' to be screamed by disgruntled
'           players;)

Dim iRetVal As Integer

'Both sprites must be active
If Not (gtSpr(viId1).iActive And gtSpr(viId2).iActive) Then Exit Function

'Default to a collision
iRetVal = True

'Check if collision is impossible
If ((gtSpr(viId2).iX + gtSpr(viId2).iW - 1) < gtSpr(viId1).iX) Then iRetVal = False
If ((gtSpr(viId2).iY + gtSpr(viId2).iH - 1) < gtSpr(viId1).iY) Then iRetVal = False
If (gtSpr(viId2).iX > (gtSpr(viId1).iX + gtSpr(viId1).iW - 1)) Then iRetVal = False
If (gtSpr(viId2).iY > (gtSpr(viId1).iY + gtSpr(viId1).iH - 1)) Then iRetVal = False

'Return success
iSprCollision = iRetVal

End Function

Function iSprCollisionRange (ByVal viId As Integer, ByVal viStart As Integer, ByVal viEnd As Integer) As Integer

'=========================================================
'Purpose    To determine if two sprites have collided
'Entry      viId - Index of first sprite
'           viStart - Index of first sprite in range
'           viEnd - Index of last sprite in range
'Exit       False if no collisions encountered, else
'           returns the ID+1 of the sprite hit
'Comments   This uses a crude overlapping rectangles
'           algorithm

Dim iId As Integer
Dim iRetVal As Integer

For iId = viStart To viEnd

    'Both sprites must be active
    If gtSpr(viId).iActive And gtSpr(iId).iActive Then

	'Default to a collision
	iRetVal = iId + 1

	'Check if collision is impossible
	If ((gtSpr(iId).iX + gtSpr(iId).iW - 1) < gtSpr(viId).iX) Then iRetVal = False
	If ((gtSpr(iId).iY + gtSpr(iId).iH - 1) < gtSpr(viId).iY) Then iRetVal = False
	If (gtSpr(iId).iX > (gtSpr(viId).iX + gtSpr(viId).iW - 1)) Then iRetVal = False
	If (gtSpr(iId).iY > (gtSpr(viId).iY + gtSpr(viId).iH - 1)) Then iRetVal = False

	'Quit checking as soon as we get a hit
	If iRetVal Then Exit For
    End If

Next iId

'Return success
iSprCollisionRange = iRetVal

End Function

Function iSprGetPlayDC (ByVal viDstDc As Integer, ByVal viW As Integer, ByVal viH As Integer) As Integer

'Purpose    Creates a DC to use as the play area
'Entry      viDstDC - DC that the play area will be displayed on
'           viW - Width of play area in pixels
'           viH - Height of play area in pixels
'Exit       True if resources allocated, false otherwise
'Comments   Will exit false if a play area is already in use

Dim i As Integer
Dim iDC As Integer
Dim iBmp As Integer
Dim iRetVal As Integer

'If a play area already exists then exit now
If miGameDC Then Exit Function

'Get a DC
iDC = CreateCompatibleDC(viDstDc)

'Only proceed on success
If iDC Then

    'Get a Bitmap
    iBmp = CreateCompatibleBitmap(viDstDc, viW, viH)

    'Only proceed on success
    If iBmp Then

	'Select bitmap into DC
	iBmp = SelectObject(iDC, iBmp)

	'Save new resource handles in module variables
	miGameDC = iDC
	miGameBmp = iBmp
	miGameW = viW
	miGameH = viH

	'Flag success
	iRetVal = True

	'Clear the play area
	i = BitBlt(miGameDC, 0, 0, viW, viH, miGameDC, 0, 0, SRCERASE)
    
    Else

	'Bitmap could not be allocated, free the DC
	i = DeleteDC(iDC)
    
    End If

End If

'Return success of operation
iSprGetPlayDC = iRetVal

End Function

Function iSprLoadFrames (ByVal vsFName As String) As Integer

'=========================================================
'Purpose    Load frame details from a file
'Entry      vsFName - name of data file to load
'Exit       True if operation successful
'Comments   The data file must reside in the same
'           directory as the executable. There are no
'           checks done on the file to make sure it is a
'           frames data file ... add this yourself or wait
'           for my frame editor utility (may be a long
'           wait;)

Dim iFNum As Integer
Dim i As Integer
Dim iCount As Integer
Dim iY As Integer
Dim iW As Integer
Dim iH As Integer
Dim iOpen As Integer
Dim sFName As String
Dim tFrame As SprFrame

'Trap errros as we are accessing disk files
On Error GoTo LoadFrames_Err

'Build full filename
sFName = App.Path & "/" & vsFName

'Open specified file
iFNum = FreeFile
Open sFName For Input As #iFNum
iOpen = True

'Read number of frames
Input #iFNum, iCount

'Redimension array
ReDim gtSprFrm(iCount)

'Read one frame at a time and add to array
For i = 0 To iCount - 1
    Input #iFNum, tFrame.iX, tFrame.iY, tFrame.iW, tFrame.iH
    gtSprFrm(i) = tFrame
Next i

'Close the file
Close #iFNum

'Set success flag and exit
iSprLoadFrames = True
Exit Function

LoadFrames_Err:
MsgBox "The following error occurred in iSprLoadGfx():" & Chr$(10) & Error$(Err)

'If file open then close it
If iOpen Then Close #iFNum

Exit Function
End Function

Function iSprLoadGfx (pic As PictureBox, ByVal vsFName As String) As Integer

'=========================================================
'Purpose    Load sprite graphics from a *.BMP file into a
'           memory DC and create a mask DC for transparent
'           blitting as well
'Entry      pic - a picture box used to load the gfx
'           vsFName - Name of the *.BMP file to load
'Exit       True if file loaded and mask created
'Comments   The *.BMP file must reside in same directory
'           as the executable. All resources are released
'           if any errors occur

Dim i As Integer
Dim iW As Integer
Dim iH As Integer
Dim iGfxDC As Integer
Dim iGfxBmp As Integer
Dim iMaskDC As Integer
Dim iMaskBmp As Integer
Dim iSrcDC As Integer
Dim iRetVal As Integer
Dim lColour As Long

'Exit if gfx are already loaded.
If miGfxDC Then Exit Function

'Trap errors as we are accessing disk files
On Error GoTo SprLoadGfx_Err

'Make sure picture box is configured as required
pic.AutoRedraw = True
pic.AutoSize = True
pic.Visible = False

'Load gfx into a temporary Picture Box
pic.Picture = LoadPicture(App.Path & "/" & vsFName)

'Get useful picture box properties into locals
iW = pic.Width \ Screen.TwipsPerPixelX
iH = pic.Height \ Screen.TwipsPerPixelY
iSrcDC = pic.hDC

'Allocate a DC
iGfxDC = CreateCompatibleDC(miGameDC)

'Allocate a Bitmap
If iGfxDC Then
    iGfxBmp = CreateCompatibleBitmap(iSrcDC, iW, iH)

    'Copy the gfx from the PictureBox into the DC
    If iGfxBmp Then
	iGfxBmp = SelectObject(iGfxDC, iGfxBmp)
	i = BitBlt(iGfxDC, 0, 0, iW, iH, iSrcDC, 0, 0, SRCCOPY)

	'Allocate a DC for masks
	iMaskDC = CreateCompatibleDC(miGameDC)

	'Allocate a Bitmap for masks
	If iMaskDC Then
	    iMaskBmp = CreateBitmap(iW, iH, 1, 1, ByVal 0&)

	    'Create masks
	    If iMaskBmp Then
		iMaskBmp = SelectObject(iMaskDC, iMaskBmp)
		lColour = SetBkColor(iGfxDC, QBColor(0))
		i = BitBlt(iMaskDC, 0, 0, iW, iH, iGfxDC, 0, 0, SRCCOPY)
		lColour = SetBkColor(iGfxDC, lColour)

		'Flag success and store variables
		iRetVal = True
		miGfxDC = iGfxDC
		miGfxBmp = iGfxBmp
		miMaskDC = iMaskDC
		miMaskBmp = iMaskBmp
	    End If
	End If
    End If
End If

'If we failed for some reason then free all resources
If Not iRetVal Then

    'Free sprite gfx bitmap
    If iGfxBmp Then
	iGfxBmp = SelectObject(iGfxDC, iGfxBmp)
	i = DeleteObject(iGfxBmp)
    End If

    'Free sprite gfx DC
    If iGfxDC Then i = DeleteDC(iGfxDC)

    'Free mask bitmap
    If iMaskBmp Then
	iMaskBmp = SelectObject(iMaskDC, iMaskBmp)
	i = DeleteObject(iMaskBmp)
    End If

    'Free mask DC
    If iMaskDC Then i = DeleteDC(iMaskDC)

End If

'Set return code and exit
iSprLoadGfx = iRetVal
Exit Function

'Simple error handler
SprLoadGfx_Err:

'Inform user of the error
MsgBox "The following error occurred in iSprLoadGfx():" & Chr$(10) & Error$(Err)

'In case this was not a file error free resources
    
'Free sprite gfx bitmap
If iGfxBmp Then
    iGfxBmp = SelectObject(iGfxDC, iGfxBmp)
    i = DeleteObject(iGfxBmp)
End If

'Free sprite gfx DC
If iGfxDC Then i = DeleteDC(iGfxDC)

'Free mask bitmap
If iMaskBmp Then
    iMaskBmp = SelectObject(iMaskDC, iMaskBmp)
    i = DeleteObject(iMaskBmp)
End If

'Free mask DC
If iMaskDC Then i = DeleteDC(iMaskDC)

Exit Function
End Function

Sub SprActivateSprite (ByVal viId As Integer, ByVal viX As Integer, ByVal viY As Integer)

'=========================================================
'Purpose    Activates a sprite and makes initial background
'           save
'Entry      viId - Index of sprite to activate
'           viX - X pixel coord to display sprite at
'           viY - Y pixel coord to display sprite at
'Comments

Dim i As Integer

'Exit if sprite is not allocated
If Not gtSpr(viId).iInUse Then Exit Sub

'Set the sprites X, Y coordinates
gtSpr(viId).iX = viX
gtSpr(viId).iY = viY

'Take a copy of background at (X,Y) where sprite will be displayed
i = BitBlt(gtSpr(viId).iSaveDC, 0, 0, gtSpr(viId).iW, gtSpr(viId).iH, miGameDC, viX, viY, SRCCOPY)

'Mark the sprite as active
gtSpr(viId).iActive = True

End Sub

Sub SprAnimAuto ()

'=========================================================
'Purpose    Animate all active sprites that have iAnimAuto
'           set True
'Entry      None
'Comments   All animation sequences are assumed to be
'           cyclic.

Dim i As Integer
Dim iId As Integer
Dim iStart As Integer
Dim iEnd As Integer
Dim iFrame As Integer

'Get dimensions of the sprite array
iStart = LBound(gtSpr)
iEnd = UBound(gtSpr)

'For each active entry in gtSpr() with iAnimAuto set true
For iId = iStart To iEnd
    If gtSpr(iId).iActive And gtSpr(iId).iAnimAuto Then
	
	iFrame = gtSpr(iId).iFrame
	
	'Decrease iAnimCount
	gtSpr(iId).iAnimCount = gtSpr(iId).iAnimCount - 1

	'If iAnimCount is zero then
	If gtSpr(iId).iAnimCount = 0 Then
	    
	    'reset iAnimCount to value in iAnimRate
	    gtSpr(iId).iAnimCount = gtSpr(iId).iAnimRate
	    
	    'bump iFrame
	    iFrame = iFrame + 1

	    'If iFrame > iLastFrame then reset iFrame to iFirstFrame
	    If iFrame > gtSpr(iId).iLastFrame Then iFrame = gtSpr(iId).iFirstFrame
	End If
	
	'set iFrameX and iFrameY to values in gtSprFrm(iFrame)
	gtSpr(iId).iFrame = iFrame
	gtSpr(iId).iFrameX = gtSprFrm(iFrame).iX
	gtSpr(iId).iFrameY = gtSprFrm(iFrame).iY
    End If
Next iId

End Sub

Sub SprAnimNextFrame (ByVal viId As Integer)

'=========================================================
'Purpose    To move a sprite one frame forward in its
'           animation sequence
'Entry      viId - Index of sprite to animate
'Comments   All animation sequences are assumed to be
'           cyclic

Dim iFrame As Integer

'Sprite must be active
If gtSpr(viId).iActive Then
    iFrame = gtSpr(viId).iFrame
	
    'bump iFrame
    iFrame = iFrame + 1

    'If iFrame > iLastFrame then reset iFrame to iFirstFrame
    If iFrame > gtSpr(viId).iLastFrame Then iFrame = gtSpr(viId).iFirstFrame
	
    'set iFrameX and iFrameY to values in gtSprFrm(iFrame)
    gtSpr(viId).iFrame = iFrame
    gtSpr(viId).iFrameX = gtSprFrm(iFrame).iX
    gtSpr(viId).iFrameY = gtSprFrm(iFrame).iY
End If

End Sub

Sub SprAnimPrevFrame (ByVal viId As Integer)

'=========================================================
'Purpose    To move a sprite one frame forward in its
'           animation sequence
'Entry      viId - Index of sprite to animate
'Comments   All animation sequences are assumed to be
'           cyclic

Dim iFrame As Integer

'Sprite must be active
If gtSpr(viId).iActive Then
    iFrame = gtSpr(viId).iFrame
	
    'bump iFrame
    iFrame = iFrame - 1

    'If iFrame < iFirstFrame then reset iFrame to iLastFrame
    If iFrame < gtSpr(viId).iFirstFrame Then iFrame = gtSpr(viId).iLastFrame
    
    'set iFrameX and iFrameY to values in gtSprFrm(iFrame)
    gtSpr(viId).iFrame = iFrame
    gtSpr(viId).iFrameX = gtSprFrm(iFrame).iX
    gtSpr(viId).iFrameY = gtSprFrm(iFrame).iY
End If

End Sub

Sub SprAnimSetFrame (ByVal viId As Integer, ByVal viFrame As Integer)

'=========================================================
'Purpose    To set the frame used by a sprite
'Entry      viId - Index of sprite to animate
'           viFrame - frame to use
'Comments   The frames specified can be any frame, there
'           is no requirement that it be a member of the
'           animation sequence of the sprite

'Sprite must be active
If gtSpr(viId).iActive Then
    
    'set iFrameX and iFrameY to values in gtSprFrm(iFrame)
    gtSpr(viId).iFrame = viFrame
    gtSpr(viId).iFrameX = gtSprFrm(viFrame).iX
    gtSpr(viId).iFrameY = gtSprFrm(viFrame).iY
End If

End Sub

Sub SprAnimSetFrameRel (ByVal viId As Integer, ByVal viRel As Integer)

'=========================================================
'Purpose    To set the frame used by a sprite from its
'           animation sequence
'Entry      viId - Index of sprite to animate
'           viRel - Offset to required frame, 0=iFirstFrame
'                   , 1=iFirstFrame+1,....
'Comments   The is no bounds checking done on the offset,
'           make sure it stays within iFirstFrame to
'           iLastFrame range!

Dim iFrame As Integer

'Sprite must be active
If gtSpr(viId).iActive Then
    
    iFrame = gtSpr(viId).iFirstFrame + viRel

    'set iFrameX and iFrameY to values in gtSprFrm(iFrame)
    gtSpr(viId).iFrame = iFrame
    gtSpr(viId).iFrameX = gtSprFrm(iFrame).iX
    gtSpr(viId).iFrameY = gtSprFrm(iFrame).iY
End If

End Sub

Sub SprDeactivateSprite (ByVal viId As Integer)

'=========================================================
'Purpose    Deactivates a sprite
'Entry      viId - Index of sprite to deactivate
'Comments

'Exit if sprite is not allocated
If Not gtSpr(viId).iActive Then Exit Sub

'Mark sprite as inactive
gtSpr(viId).iActive = False

End Sub

Sub SprDraw ()

'=========================================================
'Purpose    Save background and draw all sprites
'Entry      None
'Comments   Call this in game loop AFTER activating,
'           moving, deactivating sprites.

Dim i As Integer
Dim iX As Integer
Dim iY As Integer
Dim iW As Integer
Dim iH As Integer
Dim iGfxX As Integer
Dim iGfxY As Integer
Dim iSaveDC As Integer
Dim iStart As Integer
Dim iEnd As Integer
Dim iId As Integer

'Exit if there is no play area DC
If miGameDC = 0 Then Exit Sub

'Get size of array
iStart = LBound(gtSpr)
iEnd = UBound(gtSpr)

'For each active sprite in gtSpr(), working forwards through the array
For iId = iStart To iEnd
    If gtSpr(iId).iActive Then
	
	'Get working data
	iX = gtSpr(iId).iX
	iY = gtSpr(iId).iY
	iGfxX = gtSpr(iId).iFrameX
	iGfxY = gtSpr(iId).iFrameY
	iW = gtSpr(iId).iW
	iH = gtSpr(iId).iH
	iSaveDC = gtSpr(iId).iSaveDC

	'Copy data from background into sprites save DC where sprite is to be displayed
	i = BitBlt(iSaveDC, 0, 0, iW, iH, miGameDC, iX, iY, SRCCOPY)
	
	'Copy sprite data into background using transparent blitting
	i = BitBlt(miGameDC, iX, iY, iW, iH, miMaskDC, iGfxX, iGfxY, SRCAND)
	i = BitBlt(miGameDC, iX, iY, iW, iH, miGfxDC, iGfxX, iGfxY, SRCINVERT)
	
    End If
Next iId

End Sub

Sub SprFreeAll ()

'Purpose    Free all sprite engine resources
'Entry      None
'Exit       None
'Comments   Call this just before program termination

'Free resources used by sprites
SprFreeAllSprites

'Free resources used to hold sprite graphics and masks
SprFreeGfx

'Free the play area DC
SprFreePlayDC

End Sub

Sub SprFreeAllSprites ()

'=========================================================
'Purpose    Free resources used by all sprites and mark
'           them as available
'Entry      None
'Comments

Dim i As Integer
Dim iStart As Integer
Dim iEnd As Integer

'Get dimensions of the sprite array
iStart = LBound(gtSpr)
iEnd = UBound(gtSpr)

'Free all the sprites
For i = iStart To iEnd
    SprFreeSprite i
Next i

End Sub

Sub SprFreeGfx ()

'=========================================================
'Purpose    Free resources used to hold sprite graphics
'           and masks
'Entry      None

Dim i As Integer

'Free sprite gfx bitmap
If miGfxBmp Then
    miGfxBmp = SelectObject(miGfxDC, miGfxBmp)
    i = DeleteObject(miGfxBmp)
End If

'Free sprite gfx DC
If miGfxDC Then i = DeleteDC(miGfxDC)

'Free mask bitmap
If miMaskBmp Then
    miMaskBmp = SelectObject(miMaskDC, miMaskBmp)
    i = DeleteObject(miMaskBmp)
End If

'Free mask DC
If miMaskDC Then i = DeleteDC(miMaskDC)

End Sub

Sub SprFreePlayDC ()

'=========================================================
'Purpose    Release resources used for the game play area
'Entry      None
'Comments

Dim i As Integer

'If Bitmap exists select it out of DC and free it
If miGameBmp Then
    miGameBmp = SelectObject(miGameDC, miGameBmp)
    i = DeleteObject(miGameBmp)
    miGameBmp = 0
End If

'If DC exists free it
If miGameDC Then
    i = DeleteDC(miGameDC)
    miGameDC = 0
End If

End Sub

Sub SprFreeSprite (ByVal viId As Integer)

'=========================================================
'Purpose    Free resources used by a sprite and mark it
'           as available
'Entry      viId - Index value of the sprite to free
'Comments

Dim i As Integer

'If this sprite is not allocated then exit
If Not gtSpr(viId).iInUse Then Exit Sub

'If there is a save Bitmap select it out of DC and free it
If gtSpr(viId).iSaveBmp Then
    gtSpr(viId).iSaveBmp = SelectObject(gtSpr(viId).iSaveDC, gtSpr(viId).iSaveBmp)
    i = DeleteObject(gtSpr(viId).iSaveBmp)
End If

'If there is a save DC free it
If gtSpr(viId).iSaveDC Then i = DeleteDC(gtSpr(viId).iSaveDC)

'Mark the sprite as not allocated
gtSpr(viId).iInUse = False
gtSpr(viId).iActive = False
gtSpr(viId).iSaveBmp = 0    'Saftey measure!
gtSpr(viId).iSaveDC = 0

End Sub

Sub SprMoveSprite (ByVal viId As Integer, ByVal viX As Integer, ByVal viY As Integer)

'=========================================================
'Purpose    To move a sprite to absolute coordinates
'Entry      viId - Spride index
'           viX - X pixel coordinate
'           viY - Y pixel coordinate
'Comments

'Exit if sprite is not active
If Not gtSpr(viId).iActive Then Exit Sub

'Update sprites iX and iY fields
gtSpr(viId).iX = viX
gtSpr(viId).iY = viY

End Sub

Sub SprMoveSpriteRel (ByVal viId As Integer, ByVal viDx As Integer, ByVal viDy As Integer)

'=========================================================
'Purpose    To move a sprite relative to its current
'           position
'Entry      viId - Spride index
'           viDx - X pixel offset (+ or -)
'           viDy - Y pixel offset (+ or -)
'Comments

'Exit if sprite is not active
If Not gtSpr(viId).iActive Then Exit Sub

'Update sprites iX and iY fields
gtSpr(viId).iX = gtSpr(viId).iX + viDx
gtSpr(viId).iY = gtSpr(viId).iY + viDy

End Sub

Sub SprRestore ()

'=========================================================
'Purpose    Restores background where all sprites have
'           been displayed.
'Entry      None
'Comments   Call this in game loop BEFORE deactivating
'           sprites.

Dim i As Integer
Dim iStart As Integer
Dim iEnd As Integer
Dim iId As Integer

'Exit if there is no play area DC
If miGameDC = 0 Then Exit Sub

'Get size of array
iStart = LBound(gtSpr)
iEnd = UBound(gtSpr)

'For each active sprite in gtSpr(), working backwards through the array
For iId = iEnd To iStart Step -1
    
    'Copy data from sprites save DC into the background where sprite was displayed
    If gtSpr(iId).iActive Then i = BitBlt(miGameDC, gtSpr(iId).iX, gtSpr(iId).iY, gtSpr(iId).iW, gtSpr(iId).iH, gtSpr(iId).iSaveDC, 0, 0, SRCCOPY)

Next iId

End Sub

Sub SprShowPlayDC (ByVal viDstDc As Integer, ByVal viX As Integer, ByVal viY As Integer)

'=========================================================
'Purpose    Display the play area
'Entry      viDstDC - Target DC to copy play area into
'           viX - X pixel coord in target DC
'           viY - Y pixel coord in target DC

Dim i As Integer

'If play DC exists then copy it to the specified DC at specified position.
If miGameDC Then i = BitBlt(viDstDc, viX, viY, miGameW, miGameH, miGameDC, 0, 0, SRCCOPY)

End Sub

